clear
octave=0

if (octave==1)
    load inverseProblem.oct
    fig=0;
else
    load inverseProblem.mat
    fig=1;
end
fig=1;

%===================
% general parameters
%===================
coupling=0; %joint inversion or not
smoothS=10; %correlation distance for slowness covariance matrix
smoothR=10; %correlation distance for density covariance matrix
sigmaS=1.e-2;  %Slowness sigma (fail with lower values 1.e-4; 1.e-5)
sigmaRho=1000; %density sigma (fail with lower values 100)

sigmaT=0.05; % std error on travel time, sec
%sigmaT=1.e5 % to invert no ttime data, set to 1.e5
sigmaG=0.05; % std error on gravity measurement, mugal
%sigmaG=1.e5;% to invert no gravity data, set to 1.e5

sigmaCoupling=sigmaS/150; % setting for np=50; with lower values (10) doesn't couple
sigmaCoupling=3*sigmaS;   %setting for np=20;
%===================
% end general parameters
%===================


%
% we want to invert the system:
%
% m-mprior = (GT * Cdinv * G  + Cminv)^-1 * GT * Cdinv * (dobs-G*mprior)
%
%
% We need to build G, Cdinv and Cminv
%
% The data consists on ns travel times and 2*ns gravity data
%
% The parameters consist in 2*np slowness and 2*np volumic masses

% ===============
% Data vector 'D'
% ===============
% it was already set in setup.m script
%
D=data;
ndata=3*ns;

% ===============
% parameter vector 'P'
% ===============
% We have the true solution in modeltrue variable, let initialise the
% solution vector and a prior hypothesis
%
% 2*ns first values are slowness
% 2*ns last values are density - 2700

P=zeros(4*np,1);
Pprior=P;
nparam=4*np;

% ===============
% matrix G
% ===============
% is a conctanenation of TtimeDyke and Gr
G=zeros(ndata,nparam);
G(1:ns,1:2*np)=TtimeDyke;
G(ns+1:3*ns,2*np+1:4*np)=Gr;

% ===============
% Covariance matrix on data
% =============== 
%
% all data are independant
%


Cd=zeros(ndata,ndata);
Cd(1:ns,1:ns)=eye(ns)*sigmaT*sigmaT;
Cd(ns+1:end,ns+1:end)=eye(2*ns)*sigmaG*sigmaG;
CDinv=inv(Cd);

% ===============
% ===============
% Covariance matrix on parameter
% =============== 
% ===============
%
% S=1/V
% dS=-dV/V2

% ==================
% slowness parameters
% ==================

%
% If we want to smooth the parameters, the probleme is that 
% the two dykes are independant
% each dyke contains np parameters
%
% smooth on parameters
cminv=formCMinv(np,smoothS);
ZFILL=zeros(np,np);
CMinvS=vertcat(horzcat(cminv/sigmaS/sigmaS,ZFILL),horzcat(ZFILL,cminv/sigmaS/sigmaS));


% ==================
% density parameters
% ==================

%
% If we want to smooth the parameters, the probleme is that 
% the two dykes are independant
% each dyke contains np parameters
%
% smooth on parameters
cminv=formCMinv(np,smoothR);
CMinvR=vertcat(horzcat(cminv/sigmaRho/sigmaRho,ZFILL),horzcat(ZFILL,cminv/sigmaRho/sigmaRho));

% ==================
% gather parameters
% ==================
ZFILL=zeros(2*np,2*np);
CMinv=vertcat(horzcat(CMinvS,ZFILL),horzcat(ZFILL,CMinvR));
clear ZFILL CMinvS CMinvR

% ==================
% coupling
% ==================
%
% each value of slowness at one point is coupled with the desity value ate
% the same point by a linear relation that we introduce via the covariance
% term.
% coupling relation
% Slowness = alpha * rho (+ S0)

% V=[3000 2000 1000];
% rho=[2800 2700 2000];
% figure;plot(rho,1./V,'o',rho,polyval(polyfit(rho,1./V,1),rho))
if (coupling==1)
alpha=8.e-7;

for i=1:2*ns % loop over slowness parameter
    j=i+2*ns; %index of colocated density parameter
    CMinv(i,j)=alpha/sigmaCoupling/sigmaCoupling;
    CMinv(j,i)=CMinv(i,j);
end
end

%==================
%==================

% solution of the inverse problem
A=transpose(G)*CDinv;
CMpost=inv(A*G+CMinv);

% compute parameters
M = CMpost*A*D;

% extract slowness, velocity and density
Sest=M(1:2*np);
Vest=1./Sest;
Rhoest=M(2*np+1:4*np);
if (fig==1)
figure;subplot(2,1,1);plot(xp,Vest,'.');title('inverted velocity');
subplot(2,1,2);plot(xp,Rhoest,'.');title('inverted density')
end

% computed data
Dc=G*M;
residus=abs(Dc-D);

%extract ttime and gravity
ttimeCalc=Dc(1:ns);%+ttimeCone;
LeftgravityCalc=Dc(ns+1:2*ns);%+G0(1:ns);
RightGravityCalc=Dc(2*ns+1:3*ns);%+G0(ns+1:2*ns);
ttimeObs=D(1:ns);%+ttimeCone;
LeftgravityObs=D(ns+1:2*ns);
RightgravityObs=D(2*ns+1:3*ns);

if (fig==1)
figure;subplot(2,1,1);plot(y1,ttimeCalc,y1,ttimeObs);title('traveltime')
subplot(2,1,2);plot(xs(1:ns),LeftgravityCalc,xs(ns+1:2*ns),LeftgravityCalc,xs(1:ns),LeftgravityObs,xs(ns+1:2*ns),LeftgravityObs);title('gravity')
end



